@@ -25,14 +25,10 @@ import("core.base.option")
 function main(filedirs, opt)
     opt = opt or {}
     for _, filedir in ipairs(filedirs) do
-        os.tryrm(filedir)
-        if option.get("all") or opt.emptydir then
-            -- remove it if the parent directory is empty
-            local parentdir = path.directory(filedir)
-            while parentdir and os.isdir(parentdir) and os.emptydir(parentdir) do
-                os.tryrm(parentdir)
-                parentdir = path.directory(parentdir)
-            end
+        -- os.exists will return false if symlink -> not found, but we need still remove this symlink
+        if os.exists(filedir) or os.islink(filedir) then
+            -- we cannot use os.tryrm, because we need raise exception if remove failed with `uninstall --admin`
+            os.rm(filedir, {emptydirs = option.get("all") or opt.emptydir})
         end
     end
 end
